const fs = require('fs');
const byline = require('byline');

const coverageFilePath = './coverage/lcov.info';
const readmeFilePath = './README.md';
const badgeAriaLabel = 'COVERAGE BADGE';
const warnThreshold = 80;
const koThreshold = 60;
const GENERATED_AT_MESSAGE = `
> This section has been **autogenerated**, do not change by hand
> _Generated: \`${new Date().toLocaleString()}\`_
`;


const getCoverageRatio = (filePath, callback) => {
    const stream = byline(fs.createReadStream(filePath, { encoding: 'utf8' }));
    let coveredLines = 0;
    let instrumentedLines = 0;

    stream.on('data', (line) => {
        if (line.indexOf('LF:') === 0) {
            instrumentedLines += parseFloat(line.split(':')[1]);
        }
        if (line.indexOf('LH:') === 0) {
            coveredLines += parseFloat(line.split(':')[1]);
        }
    });

    stream.on('error', callback);
    stream.on('end', () => callback(null, coveredLines / instrumentedLines));
};

const makeMarkdownLink = (title, link) => `![${title}](${link})`;
const makeShieldsBadgeLink = (label, message, color, style) =>
    `https://img.shields.io/badge/${label}-${encodeURIComponent(message)}-${color}?style=${style}`;

const errorBadge = (label, message, style) => makeShieldsBadgeLink(label, message, 'red', style);
const warnBadge = (label, message, style) => makeShieldsBadgeLink(label, message, 'yellow', style);
const okBadge = (label, message, style) => makeShieldsBadgeLink(label, message, 'green', style);

function generateBadge(callback) {
    getCoverageRatio(coverageFilePath, (err, ratio) => {
        if (err) return callback(err);

        const percent = Math.round(ratio * 100);
        const badge = okBadge('coverage', percent + '%', 'flat-square');

        if (percent < koThreshold) {
            badge = errorBadge('coverage', percent + '%', 'flat-square');
        }
        if (percent < warnThreshold) {
            badge = warnBadge('coverage', percent + '%', 'flat-square');
        }

        return callback(null, badge);
    });
}



function writeBadges(badges) {
    const readmeContent = fs.readFileSync(readmeFilePath).toString();
    const readmeByLines = readmeContent.split('\n');
    const badgeInjectionStart = readmeByLines.indexOf('<!-- BADGES_START -->') + 1;
    const badgeInjectionEnd = readmeByLines.indexOf('<!-- BADGES_END -->');
    const lineCountBetween = badgeInjectionEnd - badgeInjectionStart;

    badges.push(GENERATED_AT_MESSAGE);

    readmeByLines.splice(badgeInjectionStart, lineCountBetween, badges.join('\n'));
    fs.writeFileSync(readmeFilePath, readmeByLines.join('\n'));

    console.log(`Added badges to "${readmeFilePath}"`);
}

generateBadge((err, badge) => {
    if (err) return console.error(err);
    console.log(badge);
    writeBadges([makeMarkdownLink(badgeAriaLabel, badge)]);
});